
#ifndef _NGXE_H_INCLUDED_
#define _NGXE_H_INCLUDED_

#include <ngxe_ngx.h>
#include <ngxe_timer.h>
#include <ngxe_xs.h>
#include <ngxe_connection.h>

int ngxe_initialized;

#endif /* _NGXE_H_INCLUDED_ */

